% An example backtester, that uses equity.m and invest.m
% 
% Copyright 2015 Ali N. Akansu and Mustafa U. Torun
%
% A Primer for Financial Engineering
% Financial Signal Processing and Electronic Trading
%

clear all; close all; clc;

% Parameters
delta = 0.04;          % Capital allocation ratio
r_f = 1e-4;            % Interest rate, 1 bps per day
epsilon = 4e-4;        % Cost of trade, 4 bps per transaction
initial_capital = 100; % Initial capital in USD

% Load Data and Initiate Vectors
load Data
N = length(Adjusted_Price_AAPL);
e = initial_capital * ones(N, 1); % Equity
Q = zeros(2, 1); % Investment Allocation
R_AAPL = get_return(Adjusted_Price_AAPL, 'same_size'); % Returns of AAPL
R_IBM = get_return(Adjusted_Price_IBM, 'same_size');   % Returns of IBM
Signal_AAPL = zeros(N, 1); % Trading signals for AAPL
Signal_IBM = zeros(N, 1); % Trading signals for IBM

% Generate Signals
% Here we just assume that we know when to buy long or sell short the APPL
% and IBM stocks. 
Signal_AAPL(5)   = BUY;
Signal_AAPL(38)  = SELL;
Signal_AAPL(79)  = SHORT_SELL;
Signal_AAPL(105) = BUY_TO_COVER;
Signal_AAPL(140) = BUY;
Signal_AAPL(240) = SELL;

Signal_IBM(12)  = SHORT_SELL;
Signal_IBM(35)  = BUY_TO_COVER;
Signal_IBM(41)  = BUY;
Signal_IBM(141) = SELL;
Signal_IBM(155) = SHORT_SELL;
Signal_IBM(204) = BUY_TO_COVER;

% Calculate P&L
for n=2:N
    % Initiate daily vectors
    Signal = [Signal_AAPL(n); Signal_IBM(n)];
    I = [Signal_AAPL(n) ~= NONE; Signal_IBM(n) ~= NONE];
    R = [R_AAPL(n); R_IBM(n)];
    prev_Q = Q;
    
    % Calculate the investment allocation vector
    Q = invest(Q, e(n-1), delta, Signal, R);
    
    % Calculate portfolio equity
    e(n) = equity(e(n-1), r_f, Q, prev_Q, R, I, epsilon);  
end

% Plot
plot(datenum(Dates), e - initial_capital)
datetick('x')
axis tight
ylabel('P&L')

% Measurements
sr = sharpe_ratio(e, r_f);
maxdd = max_drawdown(e);
fprintf('Sharpe Ratio = %f\n', sr);
fprintf('Annualized Sharpe Ratio = %f\n', sr * sqrt(252));
fprintf('Maximum Drawdown = %f\n', maxdd);
